/***********************************************************
 * $Id$
 *
 * FancyQT6
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQT6ASYNCBROADCASTHANDLER_H_
#define FANCYQT6ASYNCBROADCASTHANDLER_H_

#define FANCYQT6_ASYNCBROADCASTHANDLER_BEAN "Fancy::Qt6::AsyncBroadcastHandler"

#include <fancyqt6.h>
#include <fancyobject.h>

#include <fancysock3/fancysockonewayhandler.h>

class QObject;

namespace Fancy {

class Map;
namespace Sock {
class InvocationContext;
}
namespace Qt6 {

class OneWayInvocationCallEmitter;

/**
 * This object is used to dispatch a call to @c Fancy::Sock::OneWayHandler.doOneWay()
 * via Qt6's asynchronous signal delivery mechanism.
 *
 * It becomes active through registering it with the client pool, like this:
 * AsyncBroadcastHandler::ptr_t asyncBroadcastHandler = AsyncBroadcastHandler::newInstance();
 * asyncBroadcastHandler->connectOneWayInvocationCallReceived(foo, SLOT(oneWayInvocationCallReceived(Fancy::Qt6::OneWayInvocationCall::ptr_t&)));
 * remotingClientPool->registerBroadcastHandler(asyncBroadcastHandler);
 */
class FANCYQT6_API AsyncBroadcastHandler : public virtual Fancy::Sock::OneWayHandler {

public:

    typedef Fancy::ObjPtr<AsyncBroadcastHandler> ptr_t;

    /**
     * Construct a AsyncBroadcastHandler
     */
    static ptr_t newInstance();

    /**
     * A shortcut for
     * <pre>
     *  QObject::connect(this->getEmitter(),
     *                   SIGNAL(oneWayInvocationCallReceived(const Fancy::Qt6::OneWayInvocationCall::ptr_t&)),
     *                   receiver,
     *                   method);
     * </pre>
     *
     * Please note, that due to limitations of Qt's asynchronous signal/slot handling, you should
     * always include @c fancyqt6onewayinvocationcall.h in your header files and use the full
     * @c Fancy::Qt6::OneWayInvocationCall::ptr_t notation instead of i.e.
     * @c Fancy:.ObjPtr<Fancy::Qt6::OneWayInvocationCall>
     *
     * @param receiver The receiving Object, in most cases a GUI widget.
     * @param method The receiving slot which should be constructed by the @c SLOT macro.
     * @return the return value of QObject::connect()
     */
    virtual bool connectOneWayInvocationCallReceived(QObject *receiver, const char * method) const = 0;

    /**
     * @return The QObject, which's signals are emitted.
     */
    virtual OneWayInvocationCallEmitter *getEmitter() const = 0;

    /** this fullfills the interface Fancy::Sock::OneWayHandler */
    virtual void doOneWay (const Fancy::ObjPtr<Fancy::Sock::InvocationContext> &ctx,
                           const Object::ptr_t &arg,
                           const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;
};

}} // end namespaces

#endif /* FANCYQT6ASYNCBROADCASTHANDLER_H_ */
