/***********************************************************
 * $Id$
 *
 * FancyQT6
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQT6STRING_H_
#define FANCYQT6STRING_H_

#define FANCYQT6_QSTRING2PRINTABLE(s) s.toUtf8().data()

#include <fancyqt6.h>
#include <QtCore/qstring.h>
#include <fancystring.h>

namespace Fancy {

namespace Qt6 {

class StringHelper;

class FANCYQT6_API String : public virtual Fancy::String {

    QString s;
    mutable Fancy::String::ptr_t utf8;
    friend class StringHelper;

public:

	static Fancy::String::ptr_t newInstance();
	static Fancy::String::ptr_t newInstance(const QString &s);
	static Fancy::String::ptr_t newInstance(const std::string &s);
	static Fancy::String::ptr_t newInstance(const char *s);
	static Fancy::Serializable::ptr_t newSerializableInstance();

	/**
	 * Register @c Fancy::Qt6::String as the default implementation of
	 * @c Fancy::String. This method uses @c Fancy::String::registerImplementation
	 * and should be called at the very beginning of you @c main function.
	 */
	static void registerAsDefaultString();

    static QString toQString(const Fancy::Object::ptr_t &o);

    static QString toQString(const std::string &utf8);

    // This was dropped as unsafe, see doc/why_String_toPrintable_is_bad.cpp
    // You might use define FANCYQT6_QSTRING2PRINTABLE or use .toUtf8().data() directly
    //static const char * toPrintable(const QString &qs);

    static std::string toStdString(const QString &qs);

    /**
     * Format a @c std::string using a printf-style format string and stdarg
     * argument list like in @c vsnprintf.
     */
#ifdef __GNUG__
      __attribute__((format(printf,1,2)))
#endif
    static QString sprintf(const char *fmt, ...);

    /**
     * Format a @c std::string using a printf-style format string and a
     * arguments.
     */
    static QString vsprintf(const char *fmt, va_list ap);

	QString getQString() const;

	std::string getUtf8String() const;

	size_t getUtf8Length() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    void writeFields(const Fancy::ObjPtr<Fancy::Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const Fancy::ObjPtr<Fancy::Serializer> &serializer, int64_t serialVersionUID);

protected:
	String();
	String(const QString &s);
    String(const QString &s, const std::string &utf8);
	~String();
};

}
}

#endif /* FANCYQT6STRING_H_ */
